@@ -36,6 +36,10 @@ from fdroidserver.exception import MetaDataException, FDroidException
 srclibs = None
 warnings_action = None
 
+# validates usernames based on a loose collection of rules from GitHub, GitLab,
+# Liberapay and issuehunt.  This is mostly to block abuse.
+VALID_USERNAME_REGEX = re.compile(r'^[a-z\d](?:[a-z\d/._-]){0,38}$', re.IGNORECASE)
+
 
 def warn_or_exception(value):
     '''output warning or Exception depending on -W'''
@@ -65,7 +69,9 @@ app_fields = set([
     'Changelog',
     'Donate',
     'FlattrID',
+    'Liberapay',
     'LiberapayID',
+    'OpenCollective',
     'Bitcoin',
     'Litecoin',
     'Name',
@@ -109,7 +115,9 @@ yaml_app_field_order = [
     'Changelog',
     'Donate',
     'FlattrID',
+    'Liberapay',
     'LiberapayID',
+    'OpenCollective',
     'Bitcoin',
     'Litecoin',
     '\n',
@@ -168,7 +176,9 @@ class App(dict):
         self.Changelog = ''
         self.Donate = None
         self.FlattrID = None
+        self.Liberapay = None
         self.LiberapayID = None
+        self.OpenCollective = None
         self.Bitcoin = None
         self.Litecoin = None
         self.Name = None
@@ -440,10 +450,18 @@ valuetypes = {
                    r'^[0-9a-z]+$',
                    ['FlattrID']),
 
+    FieldValidator("Liberapay",
+                   VALID_USERNAME_REGEX,
+                   ['Liberapay']),
+
     FieldValidator("Liberapay ID",
                    r'^[0-9]+$',
                    ['LiberapayID']),
 
+    FieldValidator("Open Collective",
+                   r'^[0-9a-z-]+$',
+                   ['OpenCollective']),
+
     FieldValidator("HTTP link",
                    r'^http[s]?://',
                    ["WebSite", "SourceCode", "IssueTracker", "Translation", "Changelog", "Donate"]),
@@ -1481,6 +1499,7 @@ def write_plaintext_metadata(mf, app, w_comment, w_field, w_build):
     w_field_nonempty('Donate')
     w_field_nonempty('FlattrID')
     w_field_nonempty('LiberapayID')
+    w_field_nonempty('OpenCollective')
     w_field_nonempty('Bitcoin')
     w_field_nonempty('Litecoin')
     mf.write('\n')
